"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Promise = require("bluebird");
const childProcess = require("child_process");
const path = require("path");
const fs = require("fs");
const _ = require("lodash");
const cpass_1 = require("cpass");
const UrlHelper_1 = require("../../../utils/UrlHelper");
const Cache_1 = require("./../../../utils/Cache");
class OnDemand {
    constructor(_siteUrl, _authOptions) {
        this._siteUrl = _siteUrl;
        this._authOptions = _authOptions;
        this._cpass = new cpass_1.Cpass();
        _.defaults(this._authOptions, {
            force: false,
            persist: true
        });
    }
    getAuth() {
        let dataFilePath = this.getDataFilePath();
        let cookies;
        let cacheKey = this.resolveFileName();
        let cachedCookie = OnDemand.CookieCache.get(cacheKey);
        if (cachedCookie) {
            return Promise.resolve({
                headers: {
                    'Cookie': cachedCookie
                }
            });
        }
        if (!fs.existsSync(dataFilePath) || this._authOptions.force) {
            cookies = this.saveAuthData(dataFilePath);
        }
        else {
            console.log(`[node-sp-auth]: reading auth data from ${dataFilePath}`);
            cookies = JSON.parse(this._cpass.decode(fs.readFileSync(dataFilePath).toString()));
        }
        let authCookie = '';
        cookies.forEach((cookie) => {
            authCookie += `${cookie.name}=${cookie.value};`;
        });
        authCookie = authCookie.slice(0, -1);
        OnDemand.CookieCache.set(cacheKey, authCookie, OnDemand.Expiration);
        return Promise.resolve({
            headers: {
                'Cookie': authCookie
            }
        });
    }
    saveAuthData(dataPath) {
        let isWindows = (process.platform.lastIndexOf('win') === 0);
        let command = isWindows ? 'cmd.exe' : 'sh';
        let electronExecutable = this._authOptions.electron || 'electron';
        let args = `${electronExecutable} ${path.join(__dirname, 'electron/main.js')} ${this._siteUrl}`;
        const output = childProcess.execFileSync(command, [isWindows ? '/c' : '-c', args]).toString();
        let cookieRegex = /#\{([\s\S]+?)\}#/gm;
        let cookieData = cookieRegex.exec(output);
        let cookiesJson = cookieData[1].split(';#;');
        let cookies = [];
        cookiesJson.forEach((cookie) => {
            let data = cookie.replace(/(\n|\r)+/g, '').replace(/^["]+|["]+$/g, '');
            if (data) {
                let cookieData = JSON.parse(data);
                if (cookieData.httpOnly) {
                    cookies.push(cookieData);
                }
            }
        });
        if (cookies.length === 0) {
            throw new Error('Cookie array is empy');
        }
        if (this._authOptions.persist) {
            fs.writeFileSync(dataPath, this._cpass.encode(JSON.stringify(cookies)));
        }
        return cookies;
    }
    getDataFilePath() {
        let userDataFolder = this.getUserDataFolder();
        if (!fs.existsSync(userDataFolder)) {
            fs.mkdirSync(userDataFolder);
        }
        return path.join(userDataFolder, `${this.resolveFileName()}_ondemand.data`);
    }
    getUserDataFolder() {
        let platform = process.platform;
        let homepath;
        if (platform.lastIndexOf('win') === 0) {
            homepath = process.env.APPDATA || process.env.LOCALAPPDATA;
        }
        if (platform === 'darwin') {
            homepath = process.env.homepath;
            homepath = path.join(homepath, 'Library', 'Application Support');
        }
        if (platform === 'linux') {
            homepath = process.env.homepath;
        }
        if (!homepath) {
            throw new Error('Couldn\'t find the base application data folder');
        }
        return path.join(homepath, 'spauth');
    }
    resolveFileName() {
        let url = UrlHelper_1.UrlHelper.removeTrailingSlash(this._siteUrl);
        return url.replace(/[\:/\s]/g, '_');
    }
}
OnDemand.CookieCache = new Cache_1.Cache();
OnDemand.Expiration = 24 * 60 * 60;
exports.OnDemand = OnDemand;
//# sourceMappingURL=OnDemand.js.map